#!/usr/bin/env python

"""
loadRiverImages.py

main program to load png images and store into a numpy array.

This is the first script to launch for dataset creation.
It runs using MPI directives on a multi-core machine.

SEQUENCE OF SCRIPT EXECUTION:

1. (mandatory)	loadRiverImages.py
2. (mandatory)	createRiverDatasets.py
3. (mandatory)	riverAugmenting.py
4. (optional)	saveRiverImages.py

# Copyright (C) 2020-2022 DAME Project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation;
# either version 3.0 of the License, or (at your option)
# any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to
# the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301 USA
#


WHO                 WHEN         WHAT
------------------- ----------	 -------------------------------------------------
M. Brescia			19/02/2020	 Created
-----------------------------------------------------------------------------------
"""

import pandas as pd, os
#load the images
import multiprocessing
# load and display an image with Matplotlib
from matplotlib import image
#import imageio
#from PIL import Image
#from matplotlib import pyplot
import numpy as np
from tqdm import tqdm 

"""
# enable this for debug
data_dir = 'Data-debug'
in_dir = 'RiverBasedTraining-debug'
path = in_dir + os.sep + 'class-D-ND-3Mar2020.csv'

"""
data_dir = 'Data-images'
in_dir = 'exp'
path = 'class-D-ND.csv'

df = pd.read_csv(path)

filenames = np.array(df['ID'])
labels = np.array(df['is_dend'])
names = np.array(df['name'])
inputs = np.column_stack([filenames, labels, names])

#correction = np.zeros((101, 101), dtype=float)
#for i in range(101):
    #for j in range(101):
        #correction[i,j]=1.-np.e**(-((i-50)**2+(j-50)**2)/500)

def get_images_08apr(inputs):
    filename, label, name = inputs
    img = image.imread(data_dir+os.sep+name+'-'+str(filename)+'.png')
    print(name,img.shape)
    # these are already squared (540x540)
    if name in ["Susquehanna", "Umbro", "Vergas"]:
        img=img[:,:,0]
        print(name,img.shape)
        return img, label, name
    elif filename > 10103:
        img=img[:,:,0]
        print(name,img.shape)
        return img, label, name
    else:
        if not name in ["Congo", "Russian", "Vamanapuram", "FiumeGiallo", "Bunta", "Ebro", "Simeto", "Kentucky", "Chalakudy"]:
            img=img[:,:,0]  # use this to load suspected images
        #Getting the bigger side of the image
        s = max(img.shape[0:2])
        #Creating a dark square with NUMPY  
        f = np.zeros((s,s),dtype=float)
        #Getting the centering position
        ax,ay = (s - img.shape[1])//2,(s - img.shape[0])//2
        #Pasting the 'image' in a centering position
        f[ay:img.shape[0]+ay,ax:ax+img.shape[1]] = img
        print(name,f.shape)
        return f, label, name



if __name__ == '__main__':
	X, y, names = [],[],[]
	with multiprocessing.Pool(multiprocessing.cpu_count()) as p:
		with tqdm(total = inputs.shape[0]) as pbar:
			for img, label, name in p.imap(get_images_08apr, inputs):
				X.append(img)
				y.append(label)
				names.append(name)
				pbar.update()

	X = np.array(X)
	print("Initial img array dim: " + str(X.shape))
	y = np.array(y)
	X = np.expand_dims(X, axis=3)
	print("Final img array dim: " + str(X.shape))
	#saving tensors as npy files
	np.save(in_dir+os.sep+'X_riverbased.npy', X)
	np.save(in_dir+os.sep+'y_riverbased.npy', y)
	np.save(in_dir+os.sep+'name_riverbased.npy', names)
	print("Loaded img arrays: " + 'X_riverbased.npy, ' + 'y_riverbased.npy, ' + 'name_riverbased.npy')
